<?php
/*
                                MMMMMMMMM                             
                             MMMMh                                    
                           MMM                                        
                          MM                                          
                         MM        mM`                                
                         M      MM                                    
                         M     M     MMMMMMM                          
                              s             MMM                       
                                              MMMM                    
                                                MMMy                  
           M                                     -MMMM                
          MM                                       MMMMMMMs           
         MMM                                         /MMMM            
         MMMM                                         MMMM            
          MMMM                                        MMMy            
           MMMMM                                      mMMM            
            MMMMMM                  the                MMM            
             MMMMMM/             Code Beats            MMM            
               MMMMMM       http://codebeats.com       MMM            
                MMMMM                                  MMM            
                 MMMMM                                 MMM            
                  MMMM                                 MM             
                   MMM                                 MM             
                   MM                                  M              
                   MMMMMMMMMMM                        M               
                   MMMNNMMMMMMMMM                    M                
                               MMMMM                M                 
                                   mMM            m                   
                                        :                             

Code downloaded from: http://codebeats.com
by: "Tomás Jaramillo Quintero"
License: This code is under a Creative Commons Attribution 3.0 Unported License
License URL: http://creativecommons.org/licenses/by/3.0/
*/
class img {

	public $image;
	private $target;

	/*
	Stores image data in an array.
	---
	String $source: Path to the image to be copied.
	String $target: Target directory for the image.
	*/
	function __construct($source,$target=null) {
		// Retrieve image information (size, type, etc.).
		$this->image = getimagesize($source);
		if (!is_array($this->image)) {
			// Probably not an image file, or the file type is not supported by getimagesize().
			return false;
		}
		switch ($this->image['mime']) {
			case 'image/jpeg':
				$this->image['ext'] = '.jpg';
				break;
			case 'image/png':
				$this->image['ext'] = '.png';
				break;
			case 'image/gif':
				$this->image['ext'] = '.gif';
				break;
			default:
				// File type not supported by this class. Bye.
				return false;
		}
		// Name of the file without extension.
		$name = preg_replace('/\..*$/i','',basename($source));
		$this->image['name'] = $name;
		$this->image['target'] = $target;
		$this->image['source'] = $source;
		$i = 0; // Initialize counter.
		if ($this->image['target'] != null) {
			// Give a URL-friendly name to the image.
			$this->image['name'] = strtolower(preg_replace('/[^0-9a-z\-]+/i','',str_replace(' ','-',$name)));
			// Check if there's an image with the same name in the target directory.
			while (file_exists($this->image['target'] . $this->image['name'] . '.gif') or file_exists($this->image['target'] . $this->image['name'] . '.jpg') or file_exists($this->image['target'] . $this->image['name'] . '.png')) {
				// If there's already one, append a number to the end of the name, until we get an available one.
				$this->image['name'] = $name . '-' . $i;
				$i++;
			}
			$this->image['path'] = $this->image['target'] . $this->image['name'] . $this->image['ext'];
			// If the path doesn't exist.
			if (!file_exists($this->image['target'])) {
				// Create it.
				mkdir($this->image['target'], 0777, TRUE);
			}
			// Moves image from current to target location.
			if (!rename($this->image['source'],$this->image['path'])) {
				return false; // Return false on error.
			}
		} else {
			$this->image['target'] = dirname($this->image['source']) . '/';
			$this->image['path'] = $this->image['source'];
		}
		// Stores an image handler into the attribute $this->image.
		switch ($this->image['mime']) {
		case 'image/jpeg':
			$this->image['handler'] = imagecreatefromjpeg($this->image['path']);
			break;
		case 'image/png':
			$this->image['handler'] = imagecreatefrompng($this->image['path']);
			break;
		case 'image/gif':
			$this->image['handler'] = imagecreatefromgif($this->image['path']);
			break;
		}
	}

	/*
	Copies and resizes an image.
	---
	String $path: Target path for the image copy. This must include the name of the file! (e.g. c:/target_path/target_name.png)
	String $suffix: A suffix appended to name of the copy.
	Integer $width, $height: Limits OR dimensions for the new copy.
	Bool $scale: If TRUE, the image is scaled. $width and $height are taken as size limits rather than final dimensions.
	Integer $cWidth, $cHeight: These are the dimensions of the canvas of the image copy (i.e. The ultimate size of the image).
	*/
	function imgCopy($path,$suffix,$width=NULL,$height=NULL,$scale=FALSE,$x=0,$y=0,$cWidth=NULL,$cHeight=NULL) {
		// If $path is null, generate a default $path and modify it until there's no other image with the same name.
		if (is_null($path)) {
			if (is_null($suffix) == true) {
				$suffix = '';
			}
			$i = 0;
			// Check if there's an image with the same name in the target directory.
			while (file_exists($this->image['target'] . $this->image['name'] . $suffix . '.gif') or file_exists($this->image['target'] . $this->image['name'] . $suffix . '.jpg') or file_exists($this->image['target'] . $this->image['name'] . $suffix . '.png')) {
				// If there's already one, append a number to the end of the name, until we get an available one.
				$this->image['name'] = $this->image['name'] . '-' . $i;
				$i++;
			}
			$path = $this->image['target'] . $this->image['name'] . $suffix . '.png';
		} else { // This "else" can be tremendously improved! [Improve me *_* plzz]
			// If the path doesn't exist.
			if (!file_exists(dirname($path))) {
				// Create it.
				mkdir(dirname($path), 0777, TRUE);
			}
		}
		// If both $width and $height aren't numeric OR one of them is zero, there's nothing to do. Bye.
		if (((!is_numeric($width) and !is_null($width)) and (!is_numeric($height) and is_null($height))) || $width === 0 || $height === 0 || $cWidth === 0 || $cHeight === 0)
			return FALSE;
		if ($scale) { // If $scale is true, then $width and $height are limits, not final dimensions.
			// If $width is the only size limit and the image is bigger...
			if (($this->image['0'] > $width) and is_null($height) and is_numeric($width)) {
				$new_width = $width;
				$new_height = (int)(($new_width / $this->image['0']) * $this->image['1']);
			// If $height is the only size limit and the image is bigger...
			} elseif (($this->image['1'] > $height) and is_null($width) and is_numeric($height)) {
				$new_height = $height;
				$new_width = (int)(($new_height / $this->image['1']) * $this->image['0']);
			// If both $height and $width are limits and either of the image dimensions exceeds them...
			} elseif ((($this->image['0'] > $width) || ($this->image['1'] > $height)) and is_numeric($width) and is_numeric($height)) {
				// Is the ratio between the original width and original height of the image greater than
				// the ratio between $width and $height restrictions?
				if (($this->image['0'] / $this->image['1']) > ($width / $height)) {
					$new_width = $width;
					$new_height = (int)(($new_width / $this->image['0']) * $this->image['1']);
				} else {
					$new_height = $height;
					$new_width = (int)(($new_height / $this->image['1']) * $this->image['0']);
				}
			} else {
				// The image is fine. Don't resize it.
				$new_width = $this->image['0'];
				$new_height = $this->image['1'];
			}
		} else { // Don't scale the image.
			// $width and $height are the new dimensions.
			$new_width = $width;
			$new_height = $height;
		}
		 // Asign default dimensions for the canvas.
		if (is_null($cWidth)) {
			$cWidth = $new_width;
		}
		if (is_null($cHeight)) {
			$cHeight = $new_height;
		}
		$canvas = imagecreatetruecolor($cWidth,$cHeight); // Creates a black canvas with input dimensions.
		if($this->image['ext'] == '.png') {
			imagealphablending($canvas, FALSE);
			$colorTransparent = imagecolorallocatealpha($canvas, 0, 0, 0, 127);
			imagefill($canvas, 0, 0, $colorTransparent);
			// Save transparency.
			imagesavealpha($canvas, TRUE);
		} elseif($this->image['ext'] == '.gif') {
			$white = imagecolorallocate($canvas, 255, 255, 255);
			imagefill($canvas, 0, 0, $white);
		}
		$x_dest = 0;
		$y_dest = 0;
		$dst_width = $new_width;
		$dst_height = $new_height;
		$src_width = $this->image['0'];
		$src_height = $this->image['1'];
		if (!imagecopyresampled($canvas,$this->image['handler'],$x_dest,$y_dest,$x,$y,$dst_width,$dst_height,$src_width,$src_height) || !imagepng($canvas,$path,9)) {
			return FALSE; // Error merging or saving output image.
		} else {
			return TRUE; // Success!
		}
	}

	/*
	Creates thumbnails of an image.
	---
	String $path: Path where the thumbnail will be saved.
	String $width,$height: width and height of the thumbnail.
	*/
	function imgThumb($path,$suffix,$width,$height,$vertical=FALSE) {
		// If $path or $suffix are null, assign default values.
		$path = (is_null($path)?
				($this->image['target'] . $this->image['name'] . (is_null($suffix)?'':$suffix) . '.png') :
				$path);
		// Is the original height of the image greater than it width?
		if (($this->image['1'] > $this->image['0']) || $vertical==TRUE) {
			$new_width = $width;
			$new_height = (int)(($new_width/$this->image['0'])*$this->image['1']);
			$x = 0;
			$y = 0;
			// A central portion of a vertical image can be selected as well.
			// $y = (int)($this->image['1']-(($height/$width)*$this->image['0']))/2;
		} else { // It's a landscape image.
			$new_height = $height;
			$new_width = (int)(($new_height/$this->image['1'])*$this->image['0']);
			// A central portion of the image is selected.
			$x =(int)($this->image['0']-(($width/$height)*$this->image['1']))/2;
			$y = 0;
		}
		return $this->imgCopy($path,$suffix,$new_width,$new_height,FALSE,$x,$y,$width,$height);
	}

}
?>